%   Distribution code Version 1.0 by Cewu Lu Copyright 2014, The Chinese University of Hong Kong.
%
%   The Code is created based on the method described in the following paper 
%   [1] "Scale Adaptive Dictionary Learning", Cewu Lu, Jianping Shi, Jiaya Jia, IEEE Transactions on Image Processing, 2014
%    
%   if you use our code, please cite our paper.
%   The code and the algorithm are for non-comercial use only.



%% Data generation
sampleNum = 20000; % number of train sample
gtScale = 32; % the ground truth dictionary scale, in this demo case, you can chose a gtScale from 1 to 64;
[X, D_0] = data_generation(sampleNum, gtScale); % X is train sample sparsely represented by dictionary D_0

%% scale adaptive dictionary learning
max_Dict_num = 128; % guess the possible maximum dictionary scale
D = SADL(X, max_Dict_num); % scale adaptive dictionary learning

%% result demo 
disp (['the ground truth dictionary scale is; ', num2str(gtScale)]);
disp (['our guess dictionary scale is; ', num2str(size(D,2))]);
dictAtoms = dict_demo(D_0, sqrt(size(D_0,1)), size(D_0,2)); figure; imshow(dictAtoms); title('grouth truth dictionary atoms');
dictAtoms = dict_demo(D, sqrt(size(D,1)), size(D,2)); figure; imshow(dictAtoms); title('grouth truth dictionary atoms');


%% large scale test (need 30 minutes)
sampleNum = 20000;
max_Dict_num = 128;   
res = zeros(20,2); % record error
for ii = 1 : 20
    gtScale = randi(40)+24; 
    [X, D_0] = data_generation(sampleNum, gtScale);  
    D = SADL(X, max_Dict_num); % scale adaptive dictionary learning
    res(ii,:) = [size(D,2), gtScale];
end
errors = abs(res(:,1) - res(:,2));
for ii = 1 : 20
    disp(['ours ',num2str(res(ii,1)), ',  GT ',num2str(res(ii,2))])
end
disp(['our scale estimation error: mean ', num2str(mean(errors)), ', std ', num2str(std(errors))]);
